<?php
/**
 *  Download Count for WooCommerce
 *
 * @package    Download Count for WooCommerce
 * @subpackage DownloadCountWooAdmin Management screen
	Copyright (c) 2020- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$downloadcountwooadmin = new DownloadCountWooAdmin();

/** ==================================================
 * Management screen
 */
class DownloadCountWooAdmin {

	/** ==================================================
	 * Construct
	 *
	 * @since 1.00
	 */
	public function __construct() {

		add_action( 'admin_init', array( $this, 'register_settings' ) );

		add_action( 'admin_menu', array( $this, 'plugin_menu' ), 100 );
		add_filter( 'plugin_action_links', array( $this, 'settings_link' ), 10, 2 );

		if ( ! class_exists( 'TT_DownloadCountWoo_List_Table' ) ) {
			require_once dirname( __FILE__ ) . '/class-tt-downloadcountwoo-list-table.php';
		}

	}

	/** ==================================================
	 * Add a "Settings" link to the plugins page
	 *
	 * @param  array  $links  links array.
	 * @param  string $file   file.
	 * @return array  $links  links array.
	 * @since 1.00
	 */
	public function settings_link( $links, $file ) {
		static $this_plugin;
		if ( empty( $this_plugin ) ) {
			$this_plugin = 'download-count-for-woocommerce/downloadcountwoo.php';
		}
		if ( $file == $this_plugin ) {
			$donate = __( 'https://shop.riverforest-wp.info/donate/', 'download-count-for-woocommerce' );
			$links[] = '<a href="' . admin_url( 'admin.php?page=downloadcountwoo' ) . '">' . __( 'Settings' ) . '</a>';
			$links[] = '<a href="' . $donate . '" target="_blank" rel="noopener noreferrer">' . __( 'Donate to this plugin &#187;' ) . '</a>';
		}
		return $links;
	}

	/** ==================================================
	 * Settings page
	 *
	 * @since 1.00
	 */
	public function plugin_menu() {
		add_submenu_page(
			'woocommerce',
			__( 'Downloads', 'woocommerce' ) . ' ' . __( 'Count', 'download-count-for-woocommerce' ),
			__( 'Downloads', 'woocommerce' ) . ' ' . __( 'Count', 'download-count-for-woocommerce' ),
			'manage_woocommerce',
			'downloadcountwoo',
			array( $this, 'plugin_options' )
		);
	}

	/** ==================================================
	 * Settings page
	 *
	 * @since 1.00
	 */
	public function plugin_options() {

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die( esc_html__( 'You do not have sufficient permissions to access this page.' ) );
		}

		$this->options_updated();

		$scriptname = admin_url( 'admin.php?page=downloadcountwoo' );
		$downloadcountwoo_settings = get_option( 'downloadcountwoo_only_admin' );
		$dcw_settings = get_user_option( 'downloadcountwoo', get_current_user_id() );

		?>

		<div class="wrap">
		<h2>Download Count for WooCommerce</h2>

			<details>
			<summary><strong><?php esc_html_e( 'Various links of this plugin', 'download-count-for-woocommerce' ); ?></strong></summary>
			<?php $this->credit(); ?>
			</details>

			<div class="wrap">
				<details open>
				<summary><strong><?php esc_html_e( 'Settings' ); ?></strong></summary>
				<form method="post" action="<?php echo esc_url( $scriptname ); ?>">
				<?php wp_nonce_field( 'dlcwoo_set', 'downloadcountwoo_set' ); ?>
				<div style="display: block;padding:5px 5px">
				<input type="checkbox" name="display_admin_only" value="1" <?php checked( '1', get_option( 'downloadcountwoo_only_admin' ) ); ?> /><?php esc_html_e( 'Displayed on the administration screen only', 'download-count-for-woocommerce' ); ?>
				</div>
				<div style="display: block;padding:5px 5px">
				<?php esc_html_e( 'Number of items per page:' ); ?><input type="number" step="1" min="1" max="9999" style="width: 80px;" name="per_page" value="<?php echo esc_attr( $dcw_settings['per_page'] ); ?>" />
				</div>
				<?php submit_button( __( 'Save Changes' ), 'large', 'Manageset', true ); ?>
				</form>
				</details>
				<details open>
				<summary><strong><?php echo ( esc_html( __( 'Customers', 'woocommerce' ) . ' ' . __( 'Downloads', 'woocommerce' ) ) ); ?></strong></summary>
				<?php
				$download_count_woo_list_table = new TT_DownloadCountWoo_List_Table();
				$download_count_woo_list_table->prepare_items();
				$download_count_woo_list_table->display();
				?>
				</details>
			</div>

		</div>
		<?php
	}

	/** ==================================================
	 * Credit
	 *
	 * @since 1.00
	 */
	private function credit() {

		$plugin_name    = null;
		$plugin_ver_num = null;
		$plugin_path    = plugin_dir_path( __DIR__ );
		$plugin_dir     = untrailingslashit( $plugin_path );
		$slugs          = explode( '/', $plugin_dir );
		$slug           = end( $slugs );
		$files          = scandir( $plugin_dir );
		foreach ( $files as $file ) {
			if ( '.' === $file || '..' === $file || is_dir( $plugin_path . $file ) ) {
				continue;
			} else {
				$exts = explode( '.', $file );
				$ext  = strtolower( end( $exts ) );
				if ( 'php' === $ext ) {
					$plugin_datas = get_file_data(
						$plugin_path . $file,
						array(
							'name'    => 'Plugin Name',
							'version' => 'Version',
						)
					);
					if ( array_key_exists( 'name', $plugin_datas ) && ! empty( $plugin_datas['name'] ) && array_key_exists( 'version', $plugin_datas ) && ! empty( $plugin_datas['version'] ) ) {
						$plugin_name    = $plugin_datas['name'];
						$plugin_ver_num = $plugin_datas['version'];
						break;
					}
				}
			}
		}
		$plugin_version = __( 'Version:' ) . ' ' . $plugin_ver_num;
		/* translators: FAQ Link & Slug */
		$faq       = sprintf( esc_html__( 'https://wordpress.org/plugins/%s/faq', '%s' ), $slug );
		$support   = 'https://wordpress.org/support/plugin/' . $slug;
		$review    = 'https://wordpress.org/support/view/plugin-reviews/' . $slug;
		$translate = 'https://translate.wordpress.org/projects/wp-plugins/' . $slug;
		$facebook  = 'https://www.facebook.com/katsushikawamori/';
		$twitter   = 'https://twitter.com/dodesyo312';
		$youtube   = 'https://www.youtube.com/channel/UC5zTLeyROkvZm86OgNRcb_w';
		$donate    = sprintf( esc_html__( 'https://shop.riverforest-wp.info/donate/', '%s' ), $slug );

		?>
		<span style="font-weight: bold;">
		<div>
		<?php echo esc_html( $plugin_version ); ?> | 
		<a style="text-decoration: none;" href="<?php echo esc_url( $faq ); ?>" target="_blank" rel="noopener noreferrer"><?php esc_html_e( 'FAQ' ); ?></a> | <a style="text-decoration: none;" href="<?php echo esc_url( $support ); ?>" target="_blank" rel="noopener noreferrer"><?php esc_html_e( 'Support Forums' ); ?></a> | <a style="text-decoration: none;" href="<?php echo esc_url( $review ); ?>" target="_blank" rel="noopener noreferrer"><?php sprintf( esc_html_e( 'Reviews', '%s' ), $slug ); ?></a>
		</div>
		<div>
		<a style="text-decoration: none;" href="<?php echo esc_url( $translate ); ?>" target="_blank" rel="noopener noreferrer">
		<?php
		/* translators: Plugin translation link */
		echo sprintf( esc_html__( 'Translations for %s' ), esc_html( $plugin_name ) );
		?>
		</a> | <a style="text-decoration: none;" href="<?php echo esc_url( $facebook ); ?>" target="_blank" rel="noopener noreferrer"><span class="dashicons dashicons-facebook"></span></a> | <a style="text-decoration: none;" href="<?php echo esc_url( $twitter ); ?>" target="_blank" rel="noopener noreferrer"><span class="dashicons dashicons-twitter"></span></a> | <a style="text-decoration: none;" href="<?php echo esc_url( $youtube ); ?>" target="_blank" rel="noopener noreferrer"><span class="dashicons dashicons-video-alt3"></span></a>
		</div>
		</span>

		<div style="width: 250px; height: 180px; margin: 5px; padding: 5px; border: #CCC 2px solid;">
		<h3><?php sprintf( esc_html_e( 'Please make a donation if you like my work or would like to further the development of this plugin.', '%s' ), $slug ); ?></h3>
		<div style="text-align: right; margin: 5px; padding: 5px;"><span style="padding: 3px; color: #ffffff; background-color: #008000">Plugin Author</span> <span style="font-weight: bold;">Katsushi Kawamori</span></div>
		<button type="button" style="margin: 5px; padding: 5px;" onclick="window.open('<?php echo esc_url( $donate ); ?>')"><?php esc_html_e( 'Donate to this plugin &#187;' ); ?></button>
		</div>

		<?php

	}

	/** ==================================================
	 * Update wp_options table.
	 *
	 * @since 1.00
	 */
	private function options_updated() {

		if ( isset( $_POST['Manageset'] ) && ! empty( $_POST['Manageset'] ) ) {
			if ( check_admin_referer( 'dlcwoo_set', 'downloadcountwoo_set' ) ) {
				if ( isset( $_POST['display_admin_only'] ) && ! empty( $_POST['display_admin_only'] ) ) {
					update_option( 'downloadcountwoo_only_admin', 1 );
				} else {
					delete_option( 'downloadcountwoo_only_admin' );
				}
				$dcw_settings = get_user_option( 'downloadcountwoo', get_current_user_id() );
				if ( ! empty( $_POST['per_page'] ) ) {
					$dcw_settings['per_page'] = absint( $_POST['per_page'] );
				}
				update_user_option( get_current_user_id(), 'downloadcountwoo', $dcw_settings );
				echo '<div class="notice notice-success is-dismissible"><ul><li>' . esc_html__( 'Settings' ) . ' --> ' . esc_html__( 'Settings saved.' ) . '</li></ul></div>';
			}
		}

	}

	/** ==================================================
	 * Settings register
	 *
	 * @since 1.08
	 */
	public function register_settings() {

		if ( ! get_user_option( 'downloadcountwoo', get_current_user_id() ) ) {
			$dcw_tbl = array(
				'per_page' => 20,
			);
			update_user_option( get_current_user_id(), 'downloadcountwoo', $dcw_tbl );
		}

	}

}


